@@ -1,11 +1,11 @@
 """
 Functions for interacting with user and app data
 """
-
 import requests as _requests
 from . import config as _config
 from . import auth as _auth
 
+
 class user():
     """
     Get user data
@@ -34,4 +34,18 @@ def model(id: str):
     params = {'id': id}
     response = _requests.get(f'{_config.api_host}/data/model', params=params, headers={'Authorization': f'Bearer {_auth.get_access_token()}'})
     results = response.json()
-    return results
\ No newline at end of file
+    return results
+
+
+def model_data(id: str, purpose: str) -> list:
+    """
+    Get the training data for a model
+
+    :param id: Model ID
+    :param purpose: classify, extract, standardize
+    :return: Model data as a list of lists
+    """
+    params = {'model_id': id, 'type': purpose}
+    response = _requests.get(f'{_config.api_host}/user/model/train', params=params, headers={'Authorization': f'Bearer {_auth.get_access_token()}'})
+    results = response.json()
+    return results['Data']
\ No newline at end of file
